using System;
using System.ComponentModel.DataAnnotations.Schema;

namespace Jellyfin.Data.Entities
{
    public partial class TrackMetadata : Metadata
    {
        partial void Init();

        /// <summary>
        /// Default constructor. Protected due to required properties, but present because EF needs it.
        /// </summary>
        protected TrackMetadata()
        {
            Init();
        }

        /// <summary>
        /// Replaces default constructor, since it's protected. Caller assumes responsibility for setting all required values before saving.
        /// </summary>
        public static TrackMetadata CreateTrackMetadataUnsafe()
        {
            return new TrackMetadata();
        }

        /// <summary>
        /// Public constructor with required data.
        /// </summary>
        /// <param name="title">The title or name of the object.</param>
        /// <param name="language">ISO-639-3 3-character language codes.</param>
        /// <param name="_track0"></param>
        public TrackMetadata(string title, string language, DateTime dateadded, DateTime datemodified, Track _track0)
        {
            if (string.IsNullOrEmpty(title))
            {
                throw new ArgumentNullException(nameof(title));
            }

            this.Title = title;

            if (string.IsNullOrEmpty(language))
            {
                throw new ArgumentNullException(nameof(language));
            }

            this.Language = language;

            if (_track0 == null)
            {
                throw new ArgumentNullException(nameof(_track0));
            }

            _track0.TrackMetadata.Add(this);

            Init();
        }

        /// <summary>
        /// Static create function (for use in LINQ queries, etc.)
        /// </summary>
        /// <param name="title">The title or name of the object.</param>
        /// <param name="language">ISO-639-3 3-character language codes.</param>
        /// <param name="_track0"></param>
        public static TrackMetadata Create(string title, string language, DateTime dateadded, DateTime datemodified, Track _track0)
        {
            return new TrackMetadata(title, language, dateadded, datemodified, _track0);
        }

        /*************************************************************************
         * Properties
         *************************************************************************/

        /*************************************************************************
         * Navigation properties
         *************************************************************************/
    }
}

